from . import NCBI_fetch
from . import accession2taxid
from . import nnID
from . import nucDL
from . import protacc2taxid
from . import slidingwindow
from . import taxid
from . import taxid2name


# To initialize and or update databases
# from MRItaxonomy import NCBI_fetch 
# NCBI_fetch.initialize()
# from MRItaxonomy import NCBI_fetch 
# NCBI_fetch.update()